// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of whiteboard_core;

/// 白板内部加载
typedef NEWhiteboardPageLoaded = void Function();

/// 白板加入成功
typedef NEWhiteboardJoinWBSucceed = void Function();

// 获取auth信息
typedef NEWhiteboardGetAuthInfo = Future<WBAuthInfo> Function();

/// 白板内部已经离开
typedef NEWhiteboardLeave = void Function();

/// 白板内部日志输出
typedef NEWhiteboardInterceptLog = void Function(dynamic log);

/// - [code]    错误码，参见 NEMeetingError 中的定义
/// - [msg] 错误描述，可能为空
typedef NEWhiteboardErrorCallback = void Function(
    {required int code, String? msg});

/// 白板服务， 主要提供会议中白板一系列方法， eg： 会议白板
abstract class NEWhiteBoard {
  /// 获取白板实例
  static NEWhiteBoard getInstance(String ver){
    if(ver=='G1'){
      return _RoomWebWhiteBoardImp.instance;
    }
    return _RoomWebWhiteBoardImpG2.instance;
  }

  /// * [WebViewController]   javascriptController
  /// [NEWhiteboardGetAuthInfo] 在G2版本回调使用，必须传入
  /// [NEWhiteboardPageLoaded] 白板加载完成回调
  /// [NEWhiteboardJoinWBSucceed] 白板登录完成回调
  /// [NEWhiteboardErrorCallback] 白板错误fail回调
  /// [NEWhiteboardLeave] 白板内部离开回调
  /// [NEWhiteboardInterceptLog] 白板内部日志拦截回调
  JavascriptChannel initWhiteboardJavascriptChannel(
    Future<WebViewController> javascriptController, {
        NEWhiteboardGetAuthInfo? onWhiteboardGetAuthInfo,
    NEWhiteboardPageLoaded? onWhiteboardPageLoaded,
    NEWhiteboardJoinWBSucceed? onWhiteboardJoinWBSucceed,
    NEWhiteboardErrorCallback? onWhiteboardErrorCallback,
    NEWhiteboardLeave? onWhiteboardLeave,
    NEWhiteboardInterceptLog? onWhiteboardInterceptLog,
  });

  /// [RoomWBLoginInfo]  登录白板账户，需要申请对应的白板appKey.创建白板账户.
  Future<String?> login(RoomWBLoginInfo whiteBoardUserInfo);

  /// 设置默认的白板容器图标，目前为默认设置
  /// 可以在[NEWhiteboardJoinWBSucceed] 之后调用
  Future<String?> setContainerOptions({WhiteBoardContainerOptions whiteBoardContainerOptions});

  /// 获取当前白板是否已经登录
  bool isWhiteboardLogin();

  /// 获取白板地址，目前是默认地址
  String getWhiteboardUrl();

  /// 设置是否有能力开启编辑
  /// 可以在[NEWhiteboardJoinWBSucceed] 之后调用
  Future<int> enableDraw(bool enable);

  /// 获取当前编辑状态
  bool getEditWhiteBoardStatus();

  /// 释放白板
  Future<int> release();
}
