// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.
// 处理G2架构的白板逻辑

part of whiteboard_core;

class _RoomWebWhiteBoardImpG2 extends _RoomWebWhiteBoardAbstract {
  factory _RoomWebWhiteBoardImpG2() => _getInstance();

  static _RoomWebWhiteBoardImpG2 get instance => _getInstance();

  static _RoomWebWhiteBoardImpG2? _instance;

  Future<WebViewController>? _controller;

  _RoomWebWhiteBoardImpG2._internal();

  static _RoomWebWhiteBoardImpG2 _getInstance() {
    _instance ??= _RoomWebWhiteBoardImpG2._internal();
    return _instance!;
  }

  @override
  Future<String?> login(
    RoomWBLoginInfo whiteBoardUserInfo,
  ) async {
    _whiteBoardUrl = whiteBoardUserInfo.url;
    String? result;
    try {
      result =
      await evaluateJavascript(WhiteBoardWebAction.jsJoinWB,
          params: whiteBoardUserInfo);
    } catch (e) {
      return e.toString();
    }
    return result;
  }

  Future<String?> leaveWB() async {
    String? result;
    try {
        result = await evaluateJavascript(WhiteBoardWebAction.jsLeaveWB);
    } catch (e) {
      return e.toString();
    }
    return Future.value(result);
  }


  /// http://dev.yunxin.163.com/docs/product/%E4%BA%92%E5%8A%A8%E7%99%BD%E6%9D%BF/%E5%BF%AB%E9%80%9F%E5%85%A5%E9%97%A8/Javascript%E6%8E%A5%E5%85%A5
  Future<String?> evaluateJavascriptWithDirect(WebViewController controller,
      {required String target, required String action, Object? params}) async {
    return await controller.evaluateJavascript(
        'javascript:WebJSBridge({"action":"jsDirectCall","param":{"target":"$target","funcName":"$action","args":[$params]}})');
  }

  @override
  String getWhiteboardUrl() {
    return defaultWhiteBoardUrlG2;
  }

  Future<void> handleJSMessage(
    JavascriptMessage message,
      NEWhiteboardGetAuthInfo? onWhiteboardGetAuthInfo,
      NEWhiteboardPageLoaded? onWhiteboardPageLoaded,
    NEWhiteboardJoinWBSucceed? onWhiteboardJoinWBSucceed,
    NEWhiteboardErrorCallback? onWhiteboardErrorCallback,
    NEWhiteboardLeave? onWhiteboardLeave,
    NEWhiteboardInterceptLog? onWhiteboardInterceptLog,
  ) async {
    var json = jsonDecode(message.message);
    var dataFromJs = RoomWebWBBase.fromJson(json as Map<String, dynamic>);
    if (onWhiteboardInterceptLog != null){
      onWhiteboardInterceptLog.call(json);
    }
    switch (dataFromJs.action) {
      case 'webPageLoaded':
        if(onWhiteboardPageLoaded != null) onWhiteboardPageLoaded.call();
        break;
      case 'webJoinWBSucceed':
        _isLogin = true;
        if(onWhiteboardJoinWBSucceed != null) onWhiteboardJoinWBSucceed.call();
        break;
      case 'webLoginIMFailed':
      case 'webError':
      case 'webJoinWBFailed':
      case 'webCreateWBFailed':
        _isLogin = false;
        var code = (dataFromJs.param['code'] ?? -1) as int;
        var msg = (dataFromJs.param['msg'] ?? '') as String;
        if(onWhiteboardErrorCallback != null){
          onWhiteboardErrorCallback.call(code:code,msg:msg);
        }
        break;
      case 'webLeaveWB':
        _isLogin = false;
        if (onWhiteboardLeave != null) onWhiteboardLeave.call();
        break;
      case 'webGetAuth':
        if(onWhiteboardGetAuthInfo!=null) {
          WBAuthInfo wbAuthInfo = await onWhiteboardGetAuthInfo.call();
          _sendAuth(wbAuthInfo);
        } else{
          print('must be set onWhiteboardGetAuthInfo');
        }
        break;
    }
  }

  Future<String?> _sendAuth(WBAuthInfo whiteBoardAuthInfo) async {
    String? result;
    try {
      result = await evaluateJavascript(WhiteBoardWebAction.jsSendAuth,
          params: whiteBoardAuthInfo);
    } catch (e) {
      return e.toString();
    }
    return result;
  }
}
